import 'dart:io';

import 'package:docutain_sdk/docutain_sdk_document.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'docutain_sdk_platform_interface.dart';
import 'docutain_sdk_ui.dart';
import 'docutain_sdk_logger.dart';
import 'docutain_sdk_document_datareader.dart';

/// An implementation of [DocutainSdkPlatform] that uses method channels.
class MethodChannelDocutainSdk extends DocutainSdkPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('docutain_sdk');

  @override
  Future<bool> initSDK(String licenseKey) {
    //return false if value is null
    return methodChannel.invokeMethod<bool>('initSDK',
        {'licenseKey': licenseKey}).then<bool>((bool? value) => value ?? false);
  }

  @override
  Future<String> getLastError() {
    return methodChannel
        .invokeMethod<String>('getLastError')
        .then<String>((String? value) => value ?? "");
  }

  @override
  Future<File?> getTraceFile() async {
    String? tracePath =
        await methodChannel.invokeMethod<String>("getTraceFile");
    if (tracePath != null) {
      return Future<File?>.value(File(tracePath));
    } else {
      return Future<File?>.value(null);
    }
  }

  @override
  Future setLogLevel(Level level) async {
    var logLevel = <String, int>{
      'level': level.index,
    };
    return await methodChannel.invokeMethod<bool>("setLogLevel", logLevel);
  }

  @override
  Future<bool> scanDocument(DocumentScannerConfiguration scanConfig) {
    var dataToPass = <String, dynamic>{
      'allowCaptureModeSetting': scanConfig.allowCaptureModeSetting,
      'allowPageRotation': scanConfig.pageEditConfig.allowPageRotation,
      'allowPageFilter': scanConfig.pageEditConfig.allowPageFilter,
      'colorPrimaryLight': scanConfig.colorConfig.colorPrimary?.item1.value,
      'colorPrimaryDark': scanConfig.colorConfig.colorPrimary?.item2.value,
      'colorSecondaryLight': scanConfig.colorConfig.colorSecondary?.item1.value,
      'colorSecondaryDark': scanConfig.colorConfig.colorSecondary?.item2.value,
      'colorOnSecondaryLight':
          scanConfig.colorConfig.colorOnSecondary?.item1.value,
      'colorOnSecondaryDark':
          scanConfig.colorConfig.colorOnSecondary?.item2.value,
      'colorScanButtonsLayoutBackgroundLight':
          scanConfig.colorConfig.colorScanButtonsLayoutBackground?.item1.value,
      'colorScanButtonsLayoutBackgroundDark':
          scanConfig.colorConfig.colorScanButtonsLayoutBackground?.item2.value,
      'colorScanButtonsForegroundLight':
          scanConfig.colorConfig.colorScanButtonsForeground?.item1.value,
      'colorScanButtonsForegroundDark':
          scanConfig.colorConfig.colorScanButtonsForeground?.item2.value,
      'colorScanPolygonLight':
          scanConfig.colorConfig.colorScanPolygon?.item1.value,
      'colorScanPolygonDark':
          scanConfig.colorConfig.colorScanPolygon?.item2.value,
      'colorBottomBarBackgroundLight':
          scanConfig.colorConfig.colorBottomBarBackground?.item1.value,
      'colorBottomBarBackgroundDark':
          scanConfig.colorConfig.colorBottomBarBackground?.item2.value,
      'colorBottomBarForegroundLight':
          scanConfig.colorConfig.colorBottomBarForeground?.item1.value,
      'colorBottomBarForegroundDark':
          scanConfig.colorConfig.colorBottomBarForeground?.item2.value,
      'colorTopBarBackgroundLight':
          scanConfig.colorConfig.colorTopBarBackground?.item1.value,
      'colorTopBarBackgroundDark':
          scanConfig.colorConfig.colorTopBarBackground?.item2.value,
      'colorTopBarForegroundLight':
          scanConfig.colorConfig.colorTopBarForeground?.item1.value,
      'colorTopBarForegroundDark':
          scanConfig.colorConfig.colorTopBarForeground?.item2.value
    };
    return methodChannel
        .invokeMethod<bool>('scanDocument', dataToPass)
        .then<bool>((bool? value) => value ?? false);
  }

  @override
  Future<File?> writePDF(String path, String fileName, bool overwrite,
      PDFPageFormat pdfPageFormat) async {
    String? returnPath = await methodChannel.invokeMethod<String>('writePDF', {
      'path': path,
      'fileName': fileName,
      'overwrite': overwrite,
      'pdfPageFormat': pdfPageFormat.index
    });
    if (returnPath != null) {
      return Future<File?>.value(File(returnPath));
    } else {
      return Future<File?>.value(null);
    }
  }

  @override
  Future<bool> setAnalyzeConfiguration(
      AnalyzeConfiguration analyzeConfiguration) {
    var analyzeConfig = <String, dynamic>{
      'readBIC': analyzeConfiguration.readBIC,
      'readPaymentState': analyzeConfiguration.readPaymentState,
    };
    return methodChannel
        .invokeMethod<bool>("setAnalyzeConfiguration", analyzeConfig)
        .then<bool>((bool? value) => value ?? false);
  }

  @override
  Future<String> analyze() {
    return methodChannel
        .invokeMethod<String>("analyze")
        .then<String>((String? value) => value ?? "");
  }

  @override
  Future<String> getText(int pageNumber) {
    var data = <String, int>{
      'pageNumber': pageNumber,
    };
    return methodChannel
        .invokeMethod<String>("getText", data)
        .then<String>((String? value) => value ?? "");
  }

  @override
  Future<int> pageCount() {
    return methodChannel
        .invokeMethod<int>("pageCount")
        .then<int>((int? value) => value ?? 0);
  }

  // @override
  // Future<File?> writeImage(int page, String path, String fileName) async {
  //   String? returnPath = await methodChannel.invokeMethod<String>('writeImage',
  //       {'path': path, 'fileName': fileName, 'page': page});
  //   if (returnPath != null) {
  //     return Future<File?>.value(File(returnPath));
  //   } else {
  //     return Future<File?>.value(null);
  //   }
  // }
}
